/**
 * @file
 * Code for the ngApp module.
 */

var drupalUser = ['$q', function($q) {
  return $q(function(resolve, reject) {
    resolve(Drupal.settings.ngApp.user);
  });
}];

var drupalLocalStorage = ['$window', function($window) {
  var factory = {};

  factory.get = function(key) {
    return $window.localStorage.getItem(key);
  };

  factory.set = function(key, data) {
    if (data) {
      return $window.localStorage.setItem(key, data);
    }
    else {
      return $window.localStorage.removeItem(key);
    }
  };

  var init = function() {
    for (var p in Drupal.settings.ngApp.factory) {
      var value = factory.get(p);
      if (!value) {
        value = {};

        // Set the client-side UTC flush baseline to compare server-side
        // "cache last flushed" UTC to.
        if (p == 'drupalLocalStorage') {
          value.flush = new Date().getTime();
        }

        factory.set(p, JSON.stringify(value));
      }
    }
  };

  var checkFlush = function() {
    var storage = JSON.parse(factory.get('drupalLocalStorage'));

    if (Drupal.settings.ngApp.factory.drupalLocalStorage.flush) {
      var utc_server = parseInt(Drupal.settings.ngApp.factory.drupalLocalStorage.flush);
      var utc_client = parseInt(storage.flush);

      if (utc_server >= utc_client) {
        // Clear all client local storage.
        for (var p in $window.localStorage) {
          factory.set(p, null);
        }

        // Re-initialize factory vars in local storage.
        init();
      }
    }
  };

  init();
  checkFlush();

  return factory;
}];

var drupalServicesInterceptor = ['$q', function($q) {
  var factory = {};
  if (!Drupal.settings.ngApp.services) {
    return factory;
  }

  // This will happen on all HTTP requests (see module config below).
  factory.request = function(config) {
    return config;
  };

  // Forces Angular to stop processing if it received bad HTTP response.
  factory.responseError = function(response) {
    return $q.reject(response);
  };

  return factory;
}];

var drupalServices = ['$http', '$q', function($http, $q) {
  var factory = {};
  if (!Drupal.settings.ngApp.services) {
    return factory;
  }

  factory.help = function() {
    return $q
      .when($http.get('ngapp/services/help'))
      .then(function(result) {
        return result.data.services;
      });
  };

  // Set magic methods for each exposed Services resource.
  // @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind
  var stub = {
    obj: null,
    fn: function(arg) {
      var meta = this.obj;

      var path = [
        meta.path,
        arg
      ];

      path = path.join('/');

      var headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
      };

      if (Drupal.settings.ngApp.services.tokens[meta.source]) {
        headers['X-CSRF-Token'] = Drupal.settings.ngApp.services.tokens[meta.source];
        headers['X-XSRF-TOKEN'] = Drupal.settings.ngApp.services.tokens[meta.source];
      }

      return $http({
          method: 'GET',
          headers: headers,
          url: path
        })
        .then(function(result) {
          // Note: output of path is only for demo purposes; the request for
          // JSON is set via request header, not through the ".json"
          // extension. But for the endpoint path to work as a basic link
          // in the browser we're doing this:
          if (meta.source == 'restws') {
            path = path.replace('/', '.json');
          }

          return {
            path: path,
            data: result.data
          };
        });
    }
  };

  var stubBinder = stub.fn;

  for (var key in Drupal.settings.ngApp.services.resources.GET) {
    factory[key] = stubBinder.bind({
      obj: Drupal.settings.ngApp.services.resources.GET[key]
    });
  }

  return factory;
}];

var $drupalApp = angular
  .module(Drupal.settings.ngApp.name, ['ngAnimate', 'ngSanitize'])
  .factory('drupalUser', drupalUser)
  .factory('drupalLocalStorage', drupalLocalStorage)
  .factory('drupalServicesInterceptor', drupalServicesInterceptor)
  .factory('drupalServices', drupalServices)
  .config(['$httpProvider', function($httpProvider) {
    $httpProvider
      .interceptors
      .push('drupalServicesInterceptor');
  }]);
